-- Instantload- hold the instant load key during start up or on the main menu to immediately load the specified save.
--local instantLoad = "quiksave.ess"
-- Scan code for the key to hold- 44 corresponds to Z. Find a full list of scan codes here: https://mwse.readthedocs.io/en/latest/lua/guide/scancodes.html
--local instantLoadKey = 44

local config = json.loadfile("config/pg_continue_config")
if not config then
	config = {
		hideCredits = true
	}
end

--Stolen (and modified) from Sophisticated Save System. Thank you NullCascade!
--https://github.com/NullCascade/morrowind-mods/blob/master/Data%20Files/MWSE/lua/nc/save/mod_init.lua
-- Modified so only one value is returned- the FILENAME of the latest save.
local function getNewestSaveFile()
	local newestSave = nil
	local newestTimestamp = 0
	for file in lfs.dir("saves") do
		if string.endswith(file, ".ess") then
			-- Check to see if the file is newer than our current newest file.
			local lastModified = lfs.attributes("saves/" .. file, "modification")
			if lastModified > newestTimestamp then
				newestSave = file
				newestTimestamp = lastModified;
			end
		end
	end

	if newestSave ~= nil then
		-- Return the whole filename, including extension.
		return newestSave
	end
end

local function saveFileExists(save)
	for file in lfs.dir("saves") do
		if file == save then
			return true
		end
	end

	return false
end


-- Instantly load a save for testing.
local function checkInstantLoad(e)
	if saveFileExists(instantLoad) then
		tes3.loadGame(instantLoad)
	end
end
event.register("key", checkInstantLoad, {filter = instantLoadKey})

local function onLoaded(e)
	event.unregister("key", checkInstantLoad, {filter = instantLoadKey})
end
event.register("loaded", onLoaded)


local mainMenu
local function onClickContinueButton()
	-- Hide the main menu, or it flickers during the load screen.
	mainMenu.visible = false

	-- We don't error check here, since the button doesn't show up without a save file.
	-- If a user deletes their save after starting the game, they deserve a crash.
	tes3.loadGame(getNewestSaveFile())
end

local function onCreatedMenuOptions(e)
	if not e.newlyCreated then
		return
	end
	-- Don't show the continue button in game, only at the main menu.
	if not tes3.onMainMenu() then
		return
	end

	if getNewestSaveFile() == nil then
		return
	end

	mainMenu = e.element
	
	local newGameButton = mainMenu:findChild(tes3ui.registerID("MenuOptions_New_container"))
	local buttonContainer = newGameButton.parent

	local button = buttonContainer:createImageButton({
		idle = "textures/menu_continue.dds",
		over = "textures/menu_continue_over.dds",
		pressed = "textures/menu_continue_pressed.dds",
	})
	button.height = 50
	button.autoHeight = false
	button:register("mouseClick", onClickContinueButton)

	buttonContainer:reorderChildren(newGameButton, button, -1)

	--Optionally hide the credits button so we bloat the menu a little less.
	local creditsButton = mainMenu:findChild(tes3ui.registerID("MenuOptions_Credits_container"))
	creditsButton.visible = not config.hideCredits

	mainMenu.autoWidth = true
	mainMenu.autoHeight = true

	mainMenu:updateLayout()
end
event.register("uiActivated", onCreatedMenuOptions, { filter = "MenuOptions" })

--ModConfig

local modConfig = {}

function modConfig.onCreate(container)
	local pane = container:createThinBorder{}
	pane.widthProportional = 1.0
	pane.heightProportional = 1.0
	pane.paddingAllSides = 12
    pane.flowDirection = "top_to_bottom"

    local header = pane:createLabel{ text = " \n 1.0" }
	header.color = tes3ui.getPalette("header_color")
    header.borderBottom = 25

	-- Description and credits

	local txtBlock = pane:createBlock()
	txtBlock.widthProportional = 1.0
	txtBlock.autoHeight = true
	txtBlock.borderBottom = 25

    local txt = txtBlock:createLabel{}
	txt.wrapText = true
    txt.text = "     ,     .    /  .\n\n  Melchior Dahrk  Greatness7.\n\n  Petethegoat."

	-- Hide Credits

	local hideBlock = pane:createBlock()
	hideBlock.flowDirection = "left_to_right"
	hideBlock.widthProportional = 1.0
	hideBlock.autoHeight = true

	local hideLabel = hideBlock:createLabel({ text = "  :" })

	local hideButton = hideBlock:createButton({ text = config.hideCredits and tes3.getGMST(tes3.gmst.sYes).value or tes3.getGMST(tes3.gmst.sNo).value })
	hideButton.absolutePosAlignX = 1.0
	hideButton.paddingTop = 2
	hideButton.borderRight = 6
	hideButton:register("mouseClick", function(e)
		config.hideCredits = not config.hideCredits
		hideButton.text = config.hideCredits and tes3.getGMST(tes3.gmst.sYes).value or tes3.getGMST(tes3.gmst.sNo).value
	end)

    pane:updateLayout()
end

function modConfig.onClose(container)
	json.savefile("config/pg_continue_config", config, { indent = true })

	if tes3.onMainMenu() then
		-- Make sure we update the credits button visiblity.
		local creditsButton = mainMenu:findChild(tes3ui.registerID("MenuOptions_Credits_container"))
		creditsButton.visible = not config.hideCredits
	end
end

local function registerModConfig()
	mwse.registerModConfig(" ", modConfig)
end
event.register("modConfigReady", registerModConfig)
